package net.knarcraft.paidsigns.command;

import net.knarcraft.paidsigns.PaidSigns;
import net.knarcraft.paidsigns.container.PaidSign;
import net.knarcraft.paidsigns.container.PaidSignCondition;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.jetbrains.annotations.NotNull;

import java.util.Map;

/**
 * A representation of the command for listing information about paid signs
 */
public class ListCommand extends TokenizedCommand {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] args) {
        super.onCommand(sender, command, label, args);
        if (argumentSize < 1) {
            sender.sendMessage("Paid signs:");
            for (String signName : PaidSigns.getInstance().getSignManager().getAllPaidSigns().keySet()) {
                sender.sendMessage(" | " + signName);
            }
            return true;
        } else if (argumentSize < 3) {
            return parsePaidSignSelection(sender);
        }
        return false;
    }

    /**
     * Parses the given input and displays the wanted paid-sign information
     *
     * @param sender <p>The command sender to display the information to</p>
     * @return <p>True if successful. False if the input contained errors</p>
     */
    private boolean parsePaidSignSelection(CommandSender sender) {
        PaidSign paidSign = PaidSigns.getInstance().getSignManager().getPaidSign(arguments.get(0));
        if (paidSign == null) {
            sender.sendMessage("No such paid sign");
            return false;
        }
        if (argumentSize < 2) {
            displayPaidSign(sender, paidSign);
        } else {
            short signLine;
            try {
                signLine = (short) (Short.parseShort(arguments.get(1)) - 1);
            } catch (NumberFormatException exception) {
                sender.sendMessage("Invalid number given");
                return false;
            }
            if (!paidSign.getConditions().containsKey(signLine)) {
                sender.sendMessage("The paid sign you specified has no condition for line " + signLine);
                return false;
            }
            PaidSignCondition condition = paidSign.getConditions().get(signLine);
            displayPaidSignCondition(sender, paidSign.getName(), signLine, condition);
        }
        return true;
    }

    /**
     * Displays information about a paid sign condition
     *
     * @param sender    <p>The command sender to display the information to</p>
     * @param signName  <p>The name of the sign to display the condition for</p>
     * @param signLine  <p>The line the condition is for</p>
     * @param condition <p>The condition to display information about</p>
     */
    private void displayPaidSignCondition(CommandSender sender, String signName, short signLine,
                                          PaidSignCondition condition) {
        sender.sendMessage("Paid sign condition info: ");
        sender.sendMessage("Paid sign name: " + signName);
        sender.sendMessage("Condition line: " + signLine);
        sender.sendMessage("Condition match string: " + condition.getStringToMatch());
        sender.sendMessage("Execute RegEx: " + condition.executeRegex());
        sender.sendMessage("Ignore case: " + condition.ignoreCase());
        sender.sendMessage("Ignore color: " + condition.ignoreColor());
    }

    /**
     * Displays information about a paid sign
     *
     * @param sender   <p>The command sender to display the information to</p>
     * @param paidSign <p>The paid sign to display information about</p>
     */
    private void displayPaidSign(CommandSender sender, PaidSign paidSign) {
        sender.sendMessage("Paid sign info:");
        sender.sendMessage("Name: " + paidSign.getName());
        sender.sendMessage("Cost: " + paidSign.getCost());
        sender.sendMessage("Permission: " + paidSign.getPermission());
        sender.sendMessage("Ignore case: " + paidSign.getIgnoreCase());
        sender.sendMessage("Ignore color: " + paidSign.getIgnoreColor());
        sender.sendMessage("Sign conditions: ");
        Map<Short, PaidSignCondition> conditions = paidSign.getConditions();
        for (short lineIndex : conditions.keySet()) {
            sender.sendMessage(" | " + (lineIndex + 1) + ". " + conditions.get(lineIndex).getStringToMatch());
        }
    }

}
